from django.core.paginator import Paginator
from django.utils.decorators import method_decorator
from django.views import View
from django.utils.translation import get_language
from arches.app.models.models import Node, ResourceInstance, GraphModel
from arches.app.utils.response import JSONResponse
from arches.app.utils.betterJSONSerializer import JSONDeserializer
from arches import VERSION as arches_version
from arches.app.utils.decorators import user_can_read_resource
from django.db.models import Value, Case, When, Q


class RelatableResourcesView(View):
    def get(self, request, graph, node_alias):
        node_filter = Q(
            alias=node_alias,
            graph__slug=graph,
            graph__publication__isnull=False,
        )
        if arches_version >= (8, 0):
            node_filter = node_filter & Q(graph__is_active=True)

        node = Node.objects.get(node_filter)

        page_number = request.GET.get("page", 1)
        filter_term = request.GET.get("filter_term", None)
        items_per_page = request.GET.get("items", 25)
        language = get_language()
        initial_values = request.GET.getlist("initialValue", "")
        config = JSONDeserializer().deserialize(node.config.value)
        graphs = [graph["graphid"] for graph in config.get("graphs", [])]

        graph_models = GraphModel.objects.filter(graphid__in=graphs).values(
            "name", "graphid"
        )

        resources = ResourceInstance.objects.filter(graph_id__in=graphs).annotate(
            order_field=Case(
                When(resourceinstanceid__in=initial_values, then=Value(0)),
                default=Value(1),
            )
        )

        query_string = "descriptors__{}__name__icontains".format(language)

        if filter_term:
            resources = resources.filter(
                Q(**{query_string: filter_term})
                | Q(resourceinstanceid__in=initial_values)
            )

        paginator = Paginator(
            resources.order_by(
                "order_field", "descriptors__{}__name".format(get_language())
            ),
            items_per_page,
        )
        page_object = paginator.get_page(page_number)
        data = [
            {
                "resourceinstanceid": resource.resourceinstanceid,
                "display_value": resource.descriptors[language]["name"],
                "order_field": resource.order_field,
            }
            for resource in page_object
        ]

        return JSONResponse(
            {
                "graphs": graph_models,
                "current_page": paginator.get_page(page_number).number,
                "total_pages": paginator.num_pages,
                "results_per_page": paginator.per_page,
                "total_results": paginator.count,
                "data": data,
            }
        )



from arches.app.utils.betterJSONSerializer import JSONDeserializer, JSONSerializer
from django.db.models import Q
from django.views.generic import View

from arches import VERSION as arches_version
from arches.app.models import models
from arches.app.utils.response import JSONResponse

# TODO: Replace with DataTypeFactory from arches_querysets
from arches.app.datatypes.datatypes import DataTypeFactory


def serialize_card_x_node_x_widget(widget, datatype_factory):
    data = JSONDeserializer().deserialize(JSONSerializer().serialize(widget))

    data["card"] = JSONDeserializer().deserialize(
        JSONSerializer().serialize(widget.card)
    )
    del data["card_id"]

    data["node"] = JSONDeserializer().deserialize(
        JSONSerializer().serialize(widget.node)
    )
    del data["node_id"]

    data["widget"] = JSONDeserializer().deserialize(
        JSONSerializer().serialize(widget.widget)
    )
    del data["widget_id"]

    try:
        pass
        # datatype = datatype_factory.get_instance(widget.node.datatype)
        # data["config"]["defaultValue"] = datatype.get_interchange_value(
        #     data["config"].get("defaultValue", None)
        # )
    except AttributeError:
        pass

    return data


class CardXNodeXWidgetView(View):
    def get(self, request, graph_slug, node_alias):
        query = Q(node__graph__slug=graph_slug, node__alias=node_alias)

        if arches_version >= (8, 0):
            query &= Q(node__source_identifier_id__isnull=True)

        card_x_node_x_widget = (
            models.CardXNodeXWidget.objects.filter(query)
            .select_related()  # eagerly load all related objects
            .first()
        )

        if not card_x_node_x_widget:
            # Supply default widget configuration.
            nodes = models.Node.objects.filter(graph__slug=graph_slug, alias=node_alias)
            if arches_version >= (8, 0):
                nodes = nodes.filter(source_identifier=None)
            node = nodes.get()
            datatype_factory = DataTypeFactory()
            d_data_type = datatype_factory.datatypes[node.datatype]
            default_widget = d_data_type.defaultwidget
            card_x_node_x_widget = models.CardXNodeXWidget(
                node=node,
                card=node.nodegroup.cardmodel_set.first(),
                widget=default_widget,
                config=default_widget.defaultconfig,
            )

        serialized = serialize_card_x_node_x_widget(
            card_x_node_x_widget, DataTypeFactory()
        )
        return JSONResponse(serialized)


class CardXNodeXWidgetListFromNodegroupView(View):
    def get(self, request, graph_slug, nodegroup_alias):
        card_x_node_x_widgets_query = Q(
            node__graph__slug=graph_slug,
            node__nodegroup__node__alias=nodegroup_alias,
        )

        if arches_version >= (8, 0):
            card_x_node_x_widgets_query &= Q(node__source_identifier_id__isnull=True)

        saved_widget_queryset = (
            models.CardXNodeXWidget.objects.filter(card_x_node_x_widgets_query)
            .select_related()
            .order_by("sortorder")
        )

        datatype_factory = DataTypeFactory()

        saved_widgets_by_node_id = {
            saved_widget.node_id: saved_widget for saved_widget in saved_widget_queryset
        }

        node_queryset = models.Node.objects.filter(
            graph__slug=graph_slug, nodegroup__node__alias=nodegroup_alias
        )
        if arches_version >= (8, 0):
            node_queryset = node_queryset.filter(source_identifier=None)

        widget_instances = []
        for node in node_queryset:
            if node.pk in saved_widgets_by_node_id:
                widget_instances.append(saved_widgets_by_node_id[node.pk])
            else:
                datatype_for_node = datatype_factory.datatypes[node.datatype]
                default_widget_definition = datatype_for_node.defaultwidget

                if default_widget_definition is not None:  # handle semantic nodes
                    widget_instances.append(
                        models.CardXNodeXWidget(
                            node=node,
                            card=node.nodegroup.cardmodel_set.first(),
                            widget=default_widget_definition,
                            config=default_widget_definition.defaultconfig,
                        )
                    )

        serialized_data = [
            serialize_card_x_node_x_widget(widget_instance, datatype_factory)
            for widget_instance in widget_instances
        ]
        return JSONResponse(serialized_data)



from arches.app.utils.betterJSONSerializer import JSONDeserializer, JSONSerializer
from django.views.generic import View

from arches import VERSION as arches_version
from arches.app.models import models
from arches.app.utils.response import JSONResponse


class CardDataView(View):
    def get(self, request, graph_slug, nodegroup_alias):

        if arches_version < (8, 0):
            card = models.CardModel.objects.filter(
                graph__slug=graph_slug,
                nodegroup__node__alias=nodegroup_alias,
            ).get()
        else:
            node = models.Node.objects.get(alias=nodegroup_alias)
            card = models.CardModel.objects.get(
                graph__slug=graph_slug,
                nodegroup_id=node.nodegroup_id,
            )

        return JSONResponse(card)



import re
from django_hosts import patterns, host

host_patterns = patterns(
    "",
    host(
        re.sub(r"_", r"-", r"arches_component_lab"),
        "arches_component_lab.urls",
        name="arches_component_lab",
    ),
)



from django.conf import settings
from django.conf.urls.static import static
from django.conf.urls.i18n import i18n_patterns
from django.urls import include, path

from arches_component_lab.views.api.card import CardDataView
from arches_component_lab.views.api.relatable_resources import RelatableResourcesView
from arches_component_lab.views.api.widgets import (
    WidgetDataView,
    NodeDataView,
)
from arches_component_lab.views.api.card_x_node_x_widget import (
    CardXNodeXWidgetView,
    CardXNodeXWidgetListFromNodegroupView,
)

from arches_querysets.rest_framework.generic_views import (
    ArchesTileDetailView,
    ArchesTileListCreateView,
)

urlpatterns = [
    path(
        "arches-component-lab/api/relatable-resources/<slug:graph>/<slug:node_alias>",
        RelatableResourcesView.as_view(),
        name="api-relatable-resources",
    ),
    path(
        "arches-component-lab/api/widget-data/<slug:graph_slug>/<slug:node_alias>",
        CardXNodeXWidgetView.as_view(),
        name="api-card-x-node-x-widget",
    ),
    path(
        "arches-component-lab/api/widget-data/<slug:graph_slug>/<slug:node_alias>",
        WidgetDataView.as_view(),
        name="api-widget-data",
    ),
    path(
        "arches-component-lab/api/node-data/<slug:graph_slug>/<slug:node_alias>",
        NodeDataView.as_view(),
        name="api-node-data",
    ),
    path(
        "arches-component-lab/api/card-x-node-x-widget-list-from-nodegroup/<slug:graph_slug>/<slug:nodegroup_alias>",
        CardXNodeXWidgetListFromNodegroupView.as_view(),
        name="api-card-x-node-x-widget-list-from-nodegroup",
    ),
    path(
        "arches-component-lab/api/card-data/<slug:graph_slug>/<slug:nodegroup_alias>",
        CardDataView.as_view(),
        name="api-card-data",
    ),
    path(
        "arches-component-lab/api/tile/<slug:graph>/<slug:nodegroup_alias>/<uuid:pk>",
        ArchesTileDetailView.as_view(),
        name="api-tile",
    ),
    path(
        "arches-component-lab/api/tile/<slug:graph>/<slug:nodegroup_alias>",
        ArchesTileDetailView.as_view(),
        name="api-foo",
    ),
    path(
        "arches-component-lab/api/tile-list-create/<slug:graph>/<slug:nodegroup_alias>/<uuid:pk>",
        ArchesTileListCreateView.as_view(),
        name="api-tile-list-create",
    ),
]

# Ensure Arches core urls are superseded by project-level urls
urlpatterns.append(path("", include("arches.urls")))

# Adds URL pattern to serve media files during development
urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

# Only handle i18n routing in active project. This will still handle the routes provided by Arches core and Arches applications,
# but handling i18n routes in multiple places causes application errors.
if settings.ROOT_URLCONF == __name__:
    if settings.SHOW_LANGUAGE_SWITCH is True:
        urlpatterns = i18n_patterns(*urlpatterns)

    urlpatterns.append(path("i18n/", include("django.conf.urls.i18n")))



from django.db.models import Q
from django.views.generic import View

from arches import VERSION as arches_version
from arches.app.models import models
from arches.app.utils.betterJSONSerializer import JSONDeserializer, JSONSerializer
from arches.app.utils.response import JSONResponse
from arches.app.datatypes.datatypes import DataTypeFactory


# TODO: Remove this in favor of card_x_node_x_widget.py View
class WidgetDataView(View):
    def get(self, request, graph_slug, node_alias):
        query_filter = Q(
            node__graph__slug=graph_slug,
            node__alias=node_alias,
        )
        if arches_version >= (8, 0):
            query_filter = query_filter & Q(
                node__source_identifier_id__isnull=True,
            )

        card_x_node_x_widget = (
            models.CardXNodeXWidget.objects.select_related("node")
            .filter(query_filter)
            .first()
        )

        if not card_x_node_x_widget:
            # Supply default widget configuration.
            nodes = models.Node.objects.filter(graph__slug=graph_slug, alias=node_alias)
            if arches_version >= (8, 0):
                nodes = nodes.filter(source_identifier=None)
            node = nodes.get()
            datatype_factory = DataTypeFactory()
            d_data_type = datatype_factory.datatypes[node.datatype]
            default_widget = d_data_type.defaultwidget
            card_x_node_x_widget = models.CardXNodeXWidget(
                node=node,
                card=node.nodegroup.cardmodel_set.first(),
                widget=default_widget,
                config=default_widget.defaultconfig,
            )

        card_x_node_x_widget_dict = JSONDeserializer().deserialize(
            JSONSerializer().serialize(card_x_node_x_widget)
        )

        datatype = DataTypeFactory().get_instance(card_x_node_x_widget.node.datatype)
        # When dropping support for v7.6, try/except can be removed
        try:
            card_x_node_x_widget_dict["config"]["defaultValue"] = (
                datatype.get_interchange_value(
                    card_x_node_x_widget_dict["config"].get("defaultValue", None)
                )
            )
        except AttributeError:
            # Handle the case where the datatype does not have a get_interchange_value method
            pass

        return JSONResponse(card_x_node_x_widget_dict)


# TODO: Replace this with nodes.py view
class NodeDataView(View):
    def get(self, request, graph_slug, node_alias):
        node_filter = Q(
            graph__slug=graph_slug,
            alias=node_alias,
        )
        if arches_version >= (8, 0):
            node_filter = node_filter & Q(
                source_identifier_id__isnull=True,
            )
        node = models.Node.objects.get(node_filter)

        return JSONResponse(node)
